@@ -59,7 +59,7 @@
 #include	"maildir/maildiraclt.h"
 #include	"maildir/maildirnewshared.h"
 #include	"maildir/maildirinfo.h"
-#include	<unicode.h>
+#include	<courier-unicode.h>
 
 #include	"rfc2045/rfc2045.h"
 #include	"rfc822/rfc822.h"
@@ -354,7 +354,7 @@ static void list_callback(const char *f, void *vp)
 	struct list_callback_utf8 *utf8=(struct list_callback_utf8 *)vp;
 	maildir_aclt_list l;
 
-	char **fn=maildir_smapfn_fromutf7(f);
+	char **fn=maildir_smapfn_fromutf8(f);
 
 	if (!fn)
 	{
@@ -1531,7 +1531,7 @@ static int dump_hdrs(int fd, unsigned long n,
 
 	src=rfc2045src_init_fd(fd);
 	h=src ? rfc2045header_start(src, rfcp):NULL;
- 
+
 	while (h &&
 	       (rc=rfc2045header_get(h, &header, &value, get_flags)) == 0
 	       && header)
@@ -2300,7 +2300,7 @@ static void copieduid(unsigned long n, char *newname)
 
 	if ((q=strrchr(p, MDIRSEP[0])) != NULL)
 		*q=0;
-	
+
 	smapword_s(p);
 	writes("\"\n");
 }
@@ -3215,6 +3215,7 @@ void smap()
 
 	char rights_buf[40];
 
+	enabled_utf8=1;
 	imapscan_init(&current_maildir_info);
 	memset(&add_flags, 0, sizeof(add_flags));
 
@@ -3376,9 +3377,9 @@ void smap()
 
 				if (strcmp(p, "INTERNALDATE") == 0 && q)
 				{
-					add_internaldate=rfc822_parsedt(q);
-
-					if (add_internaldate)
+					if (rfc822_parsedate_chk(q,
+								 &add_internaldate)
+					    == 0)
 						okmsg="INTERNALDATE set";
 				}
 
@@ -3789,7 +3790,7 @@ void smap()
 			writes("* STATUS EXISTS=");
 			writen(infoptr->nmessages+infoptr->left_unseen);
 
-			n=infoptr->left_unseen, i;
+			n=infoptr->left_unseen;
 
 			for (i=0; i<infoptr->nmessages; i++)
 			{
@@ -4214,9 +4215,7 @@ void smap()
 
 		if (strcmp(p, "IDLE") == 0)
 		{
-			if ((p=getenv("IMAP_ENHANCEDIDLE")) == NULL
-			    || !atoi(p)
-			    || imapenhancedidle())
+			if (imapenhancedidle())
 				imapidle();
 
 			read_timeout(60);
@@ -4359,9 +4358,8 @@ void smap()
 
 					up(p);
 
-					t=rfc822_parsedt(p+13);
-
-					if (t &&
+					if (rfc822_parsedate_chk(p+13, &t)
+					    == 0 &&
 					    (dummy=applymsgset(&setdate, &t))
 					    != 0)
 						break;
